unit View;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;

type
  TViewForm = class(TForm)
    Image1: TImage;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure Image1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Image1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure Image1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private 錾 }
  public
    { Public 錾 }
    Drawing: Boolean;
    OriginPt: TPoint;
    MovePt: TPoint;
    OldOriginPt: TPoint;
    OldMovePt: TPoint;
    procedure GetScreenImage;
  end;

var
  ViewForm: TViewForm;

implementation

uses Frame;

{$R *.DFM}

procedure TViewForm.GetScreenImage;
var
  DC: HDC;
begin
  Image1.Picture.Bitmap.Width := Screen.Width;
  Image1.Picture.Bitmap.Height := Screen.Height;
  DC := GetDC(0);
  BitBlt(Image1.Canvas.Handle, 0, 0,
         Image1.Picture.Bitmap.Width, Image1.Picture.Bitmap.Height,
         DC, 0, 0, SRCCOPY);
  ReleaseDC(0, DC);
end;

procedure TViewForm.FormCreate(Sender: TObject);
var
  ViewRect: Trect;
begin
  FrameForm.GetCaptureTime;
  FrameForm.GetActiveWindowRect;
  GetScreenImage;

  Drawing := False;
  ViewRect.Left := (Image1.Picture.Width - 200) div 2;
  ViewRect.Top := (Image1.Picture.Height - 200) div 2;
  ViewRect.Right := (Image1.Picture.Width + 200) div 2;
  ViewRect.Bottom := (Image1.Picture.Height + 200) div 2;
  OriginPt := ViewRect.TopLeft;
  MovePt := ViewRect.BottomRight;
end;

procedure TViewForm.FormShow(Sender: TObject);
begin
  Image1.Canvas.Pen.Mode := pmNotXor;
  Image1.Canvas.Pen.Color := clBlue;
  Image1.Canvas.Pen.Width := 3;
  Image1.Canvas.Brush.Style := bsClear;
  Left := 0;
  Top := 0;
  Width := Image1.Picture.Width;
  Height := Image1.Picture.Height;
  Image1.Canvas.Rectangle(OriginPt.X, OriginPt.Y, MovePt.X, MovePt.Y);
end;

procedure TViewForm.Image1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then
  begin
    OldOriginPt := OriginPt;
    OldMovePt := MovePt;
    Drawing := True;
    Image1.Canvas.Rectangle(OriginPt.X, OriginPt.Y, MovePt.X, MovePt.Y);
    OriginPt := Point(X, Y);
    MovePt := OriginPt;
  end;
end;

procedure TViewForm.Image1MouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if Drawing then
  begin
    Image1.Canvas.Rectangle(OriginPt.X, OriginPt.Y, MovePt.X, MovePt.Y);
    MovePt := Point(X, Y);
    Image1.Canvas.Rectangle(OriginPt.X, OriginPt.Y, MovePt.X, MovePt.Y);
  end;
end;

procedure TViewForm.Image1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  W, H: Integer;
  SRect: Trect;
  DRect: TRect;
  CW, CH: Integer;
begin
  if Button = mbLeft then
  begin
    Drawing := False;
    if (Abs(MovePt.X - OriginPt.X) < 32) or (Abs(MovePt.Y - OriginPt.Y) < 32) then
    begin
      Image1.Canvas.Rectangle(OriginPt.X, OriginPt.Y, MovePt.X, MovePt.Y);
      OriginPt := OldOriginPt;
      MovePt := OldMovePt;
      Image1.Canvas.Rectangle(OriginPt.X, OriginPt.Y, MovePt.X, MovePt.Y);
    end;
  end
  else if Button = mbRight then
  begin
    if OriginPt.X <= MovePt.X then begin
      SRect.Left := OriginPt.X; SRect.Right := MovePt.X;
    end else begin
      SRect.Left := MovePt.X; SRect.Right := OriginPt.X;
    end;

    if OriginPt.Y <= MovePt.Y then begin
      SRect.Top := OriginPt.Y; SRect.Bottom := MovePt.Y;
    end else begin
      SRect.Top := MovePt.Y; SRect.Bottom := OriginPt.Y;
    end;

    W := SRect.Right - SRect.Left + 1;
    H := SRect.Bottom - SRect.Top + 1;

    if (W >= 32) and (H >= 32) then
    begin
      Image1.Canvas.Rectangle(OriginPt.X, OriginPt.Y, MovePt.X, MovePt.Y);
      DRect := Rect(0, 0, W, H);
      FrameForm.TurboImage1.Picture.Bitmap.Width := W;
      FrameForm.TurboImage1.Picture.Bitmap.Height := H;
      FrameForm.TurboImage1.Picture.Bitmap.Canvas.CopyRect(DRect, Image1.Canvas, SRect);
      FrameForm.ClientWidth := W;
      FrameForm.ClientHeight := H;
      CW := (FrameForm.Width - FrameForm.ClientWidth) div 2;
      CH := FrameForm.Height - FrameForm.ClientHeight - CW;
      FrameForm.Left := SRect.Left - CW;
      FrameForm.Top := SRect.Top - CH;
      if FrameForm.Left < 0 then
        FrameForm.Left := 0;
      FrameForm.Top := SRect.Top - CH;
      if FrameForm.Top < 0 then
        FrameForm.Top := 0;
      ModalResult := mrOk;
    end;
  end;
end;

procedure TViewForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  W, H: Integer;
  SRect: Trect;
  DRect: TRect;
  CW, CH: Integer;
begin
  if Key = VK_ESCAPE then
  begin
    ModalResult := mrCancel;
    Exit;
  end
  else if Key = WORD('A') then
  begin
    Image1.Canvas.Rectangle(OriginPt.X, OriginPt.Y, MovePt.X, MovePt.Y);
    OriginPt := FrameForm.ActiveWindowRect.TopLeft;
    MovePt := FrameForm.ActiveWindowRect.BottomRight;
    Image1.Canvas.Rectangle(OriginPt.X, OriginPt.Y, MovePt.X, MovePt.Y);
  end
  else if Key = WORD('S') then
  begin
    Image1.Canvas.Rectangle(OriginPt.X, OriginPt.Y, MovePt.X, MovePt.Y);
    OriginPt := Point(0, 0);
    MovePt := Point(Image1.Picture.Width - 1, Image1.Picture.Height - 1);
    Image1.Canvas.Rectangle(OriginPt.X, OriginPt.Y, MovePt.X, MovePt.Y);
  end;

  if OriginPt.X <= MovePt.X then begin
    SRect.Left := OriginPt.X; SRect.Right := MovePt.X;
  end else begin
    SRect.Left := MovePt.X; SRect.Right := OriginPt.X;
  end;

  if OriginPt.Y <= MovePt.Y then begin
    SRect.Top := OriginPt.Y; SRect.Bottom := MovePt.Y;
  end else begin
    SRect.Top := MovePt.Y; SRect.Bottom := OriginPt.Y;
  end;

  W := SRect.Right - SRect.Left + 1;
  H := SRect.Bottom - SRect.Top + 1;

  if (W >= 32) and (H >= 32) then
  begin
    Image1.Canvas.Rectangle(OriginPt.X, OriginPt.Y, MovePt.X, MovePt.Y);
    DRect := Rect(0, 0, W, H);
    FrameForm.TurboImage1.Picture.Bitmap.Width := W;
    FrameForm.TurboImage1.Picture.Bitmap.Height := H;
    FrameForm.TurboImage1.Picture.Bitmap.Canvas.CopyRect(DRect, Image1.Canvas, SRect);
    FrameForm.ClientWidth := W;
    FrameForm.ClientHeight := H;
    CW := (FrameForm.Width - FrameForm.ClientWidth) div 2;
    CH := FrameForm.Height - FrameForm.ClientHeight - CW;
    FrameForm.Left := SRect.Left - CW;
    if FrameForm.Left < 0 then
      FrameForm.Left := 0;
    FrameForm.Top := SRect.Top - CH;
    if FrameForm.Top < 0 then
      FrameForm.Top := 0;
    ModalResult := mrOk;
  end;
end;

end.

